function [Eps Sig_v Sig_h PorePres] = triaxQ8( MaterialConf, CalcConf, MeshConf )       
%Sprzysto-plastyczny model z kryterium C-M, Piotr Srokosz, 2011
    
    % *** OBLICZENIA *** 
    
    MAT = MaterialConf;
    CAL = CalcConf;
    MSH = MeshConf;
    
    %przeliczenie jednostek
    MAT.Fi = MAT.Fi .* pi ./ 180;
    MAT.Psi = MAT.Psi .* pi ./ 180;
    

    %przygotowanie wynikow
    Eps = zeros(CAL.Incr,1);
    Sig_h = zeros(CAL.Incr,1);
    Sig_v = zeros(CAL.Incr,1);
    PorePres = zeros(CAL.Incr,1);
    
    %porzadkowanie wspolrzednych wezlow
    for j=1:8
        Pnts(j,1) = MSH.Nodes( MSH.Elem.nod(j), 1 );
        Pnts(j,2) = MSH.Nodes( MSH.Elem.nod(j), 2 );
    end
        
    %stan po konsolidacji
    stress = [1 1 1 1 1 1 1 1 1; 1 1 1 1 1 1 1 1 1; 0 0 0 0 0 0 0 0 0; 1 1 1 1 1 1 1 1 1];
    stress = (CAL.ConfPres - CAL.PorePres) .* stress;
    porepres = CAL.PorePres .* ones(1,9);
    u = zeros(7,1);
    strain = zeros(4,9);
    Eps(1,1) = 0;    
    %wyniki: skladowe naprezenia i cisnienia w porach w srodkowym punkcie Gaussa
    Sig_h(1,1) = -stress(1,9);
    Sig_v(1,1) = -stress(2,9);
    PorePres(1,1) = -porepres(9);

    %macierz sprezystosci
    mnoznik = MAT.E ./ ((1 + MAT.ni) .* (1 - 2 .* MAT.ni));
    De = mnoznik .* [(1 - MAT.ni) MAT.ni 0 MAT.ni; MAT.ni (1 - MAT.ni) 0 MAT.ni; 0 0 ((1 - 2 .* MAT.ni)./2) 0; MAT.ni MAT.ni 0 (1 - MAT.ni)];
    Du = [MAT.Ke MAT.Ke 0 MAT.Ke; MAT.Ke MAT.Ke 0 MAT.Ke; 0 0 0 0; MAT.Ke MAT.Ke 0 MAT.Ke];
    D = De + Du;
    
    %macierz sztywnosci
    K = zeros(16,16);
    %osiowa symetrycznosc, obwod = 2Pi*r
    t_rad = 2 .* pi;
    for iG = 1:9        
        [B detJ r] = BQ8ax( iG, Pnts, MSH );
        BDB = B' * D * B;
        GP = GaussQ8( iG );
        k = GP.w .* detJ .* BDB .* t_rad .* r;        
        K = K + k;
    end    

    %zadane niezerowe przemieszczenia
    disps = [MSH.Disps; MSH.Disps; MSH.Disps];    

    %macierz sztywnosci po uwzglednieniu podpor
    Kc = zeros(7,7);
    Kc = [K(4,4) K(4,7) K(4,9) K(4,11:13) K(4,15); K(7,4) K(7,7) K(7,9) K(7,11:13) K(7,15); K(9,4) K(9,7) K(9,9) K(9,11:13) K(9,15); K(11:13,4) K(11:13,7) K(11:13,9) K(11:13,11:13) K(11:13,15); K(15,4) K(15,7) K(15,9) K(15,11:13) K(15,15)]; 
    
    %elementy macierzy sztywnosci mnozone przez zadane niezerowe przemieszczenia
    kf = zeros(7,3);
    kf = [K(4,6) K(4,8) K(4,10); K(7,6) K(7,8) K(7,10); K(9,6) K(9,8) K(9,10); K(11:13,6) K(11:13,8) K(11:13,10); K(15,6) K(15,8) K(15,10)];
    
    %wektor przyrostu obciazen z zadanych niezerowych przyrostow przemieszczen
    dfp = kf * disps;
    
    %przyrost obciazen przerzucony na praw stron - do sil
    dfp = -1 .* dfp;
                               
    %odwrocenie macierzy sztywnosci
    iKc = inv(Kc);
                    
    %petla uwzgledniania kolejnych przyrostow obciazen dfp
    for inc = 1:CAL.Incr-1
        %zerowania
        iter = 0;  
        dfr = zeros(7,1);
        dFr = zeros(16,1);
        depsvp = zeros(4,9); 
        
        %aktualny stan naprezenia
        sig0 = stress;
                
        %petla iteracji lepkoplastycznych        
        ctrl = 1;
        while ctrl > 0        
            
            %licznik
            iter = iter + 1;
            
            %uaktualnienie obciazen: zadanego i redystrybujacego
            df = dfp + dfr;
        
            %rozwiazanie ukladu rownan
            du = iKc * df;
            
            %norma aktualnego wektora przemieszczen
            ndu_next = norm(du);
    
            %uzupelnienie przyrostu przemieszczen o zadane wartosci
            dU = [0; 0; 0; du(1); 0; MSH.Disps; du(2); MSH.Disps; du(3); MSH.Disps; du(4:6); 0; du(7); 0];            
                
            %stan odksztalcenia i naprezenia
            for iG = 1:9
                %macierz B, jakobian i rzedna x pkt. Gaussa
                [B detJ r] = BQ8ax( iG, Pnts, MSH );
                
                %przyrost odksztalcen calkowitych
                deps(:,iG) = B * dU;
        
                %przyrost naprezen od sprezystej czesci przyrostu odksztalcen
                dsig(:,iG) = De * ( deps(:,iG) - depsvp(:,iG) );
        
                %uaktualniony stan naprezenia
                sig(:,iG) = sig0(:,iG) + dsig(:,iG);
                                
                %kryterium plastycznosci
                Fc = Fmc( sig(:,iG), MAT.Fi, MAT.c );
            
                %sprawdzenie kryterium, redystrybucja stanu naprezenia
                if Fc > 0
                    %odksztalcenia lepkoplastyczne
                                        
                    %pochodna kryterium i potencjalu plastycznego
                    [dFds dQds] = dFdQ( iG, sig, MAT.Fi, MAT.c, MAT.Psi );                    
                    
                    %predkosc odksztalcen lepkoplastycznych
                    depsvpt = Fc .* dQds;
                
                    %maksymalny krok czasowy
                    DT = 4.*(1 + MAT.ni).*(1 - 2.*MAT.ni)./(MAT.E.*(1 - 2.*MAT.ni + sin(MAT.Fi).^2));
                
                    %kumulowany przyrost odksztalcen lepkoplastycznych
                    depsvp(:,iG) = depsvp(:,iG) + DT .* depsvpt;
              
                    %przyrost naprezen lepkoplastycznych
                    dsigvp = De * ( DT .* depsvpt );
                
                    %kumulowany przyrost obciazen korygujacych stan naprezenia
                    GP = GaussQ8( iG );
                    BDdevp = B' * dsigvp;
                    dFr = dFr + detJ .* GP.w .* BDdevp .* t_rad .* r;
                end
            end
            %pierwszy warunek zatrzymania
            if iter == CAL.IterMax
                ctrl = 0;
            end
     
            %drugi warunek zatrzymania
            if iter > 1
                nd = (ndu_next - ndu_prev)./ndu_prev;
                if nd < CAL.IterTol
                    ctrl = 0;
                end
            end
            ndu_prev = ndu_next;
                
            %przygotowanie wektora obciazen korygujacych
            dfr = [dFr(4); dFr(7); dFr(9); dFr(11:13); dFr(15)];
        end
                
        %uaktualnienie stanu
        stress = sig;
        strain = strain + deps;
        u = u + du;
        
        %wyniki zwracane przez funkcje (zmieniamy znak dla wygody)
        Eps(inc+1,1) = -(MSH.Disps .* inc) ./ MSH.h;
        Sig_h(inc+1,1) = -stress(1,9);
        Sig_v(inc+1,1) = -stress(2,9);
        dpp = Du*deps(:,9);
        PorePres(inc+1,1) = -PorePres(inc+1,1) - dpp(1);
    end 
end
